' ***************************************************************************
' *                                                                         *
' * This file is named 'MPUDEMO' and was converted from the file            *
' * MPUDEMO.BAS created by By Gino Silvestri [71505,1436] for Turbo Basic.  *
' * In addition it uses GUITSOND.DLL that gives Visual Basic the ability to *
' * communicate with an installed midi card.                                *
' *                                                                         *
' * Requirements: Requires VBRUN100.DLL and GUITSOND.DLL                    *
' * Note: GUITSOND.DLL must be either in your Windows directory or          *
' * a directory on your path statement                                      *
' *                                                                         *
' * Have Fun!!                                                              *
' *                                                                         *
' * Michael Love Graves [72240,1123]                                        *
' ***************************************************************************


' ***************************************************************************
' *                        D E F I N I T I O N S                            *
' ***************************************************************************
DefInt A-Z
'
Const True = -1
Const False = 0
Const ComdPort = &H331                        ' MPU-401 Command Port on IBM
Const statport = &H331                        ' MPU-401 Status Port on IBM
Const DataPort = &H330                        ' MPU-401 Data I/O Port on IBM
Const DRR = &H40                              ' Mask for Data Read Reg. Bit
Const DSR = &H80                              ' Mask for Data Set Ready Bit
Const ACK = &HFE                              ' MPU-401 Acknowledge Response
Const maskflip = &HFF                         ' MpuWait Function Bit Mask XOR
Const MPUReset = &HFF                         ' MPU-401 Total Reset Command
Const UARTMode = &H3F                         ' MPU-401 "Dumb UART Mode"
Const NoteOff = 0                             ' 0 Velocity = Note Off
Const FirstNote = 36                          ' First note synth will play
Const LastNote = 96                           ' Last note synth will play

' ***************************************************************************
' *                       I N I T I A L I Z A T I O N                       *
' ***************************************************************************
Sub RSTMPU ()                                   ' Reset the MPU-401
	
	OUT ComdPort, MPUReset                  ' Send MPU-401 RESET Command
	a = INP(DataPort)                       ' Dummy read to clear buffer

	MpuWait statport, DRR, maskflip         ' MpuWait for port ready

	OUT ComdPort, UARTMode                  ' Set MPU-401 "Dumb UART" Mode
	a = INP(DataPort)                       ' Dummy Read to clear buffer

	MpuWait statport, DSR, maskflip         ' Wait for "UART" port ready -
						' Really crucial!!!!
End Sub

' ***************************************************************************
' *                         M A I N   P R O G R A M                         *
' ***************************************************************************
Sub MpuPlay ()

	If Form1.List1.ListIndex > 0 Then
	  MidiChannel = Form1.List1.ListIndex + 1
	Else
	  MidiChannel = 1
	End If

	If Form1.List2.ListIndex > 0 Then
	  NoteVelocity = Form1.List2.ListIndex + 1
	Else
	  NoteVelocity = 64
	End If


	Form1.text1.text = "  MPUDEMO1 playing a fast scale on MIDI Channel " + Str$(MidiChannel)


	For note = FirstNote To LastNote           ' Ascending Scale
	
	Call Playit(note)                       ' Play a note
	If AbortPlay = True Then Exit Sub
	Delay 6000                               ' Duration of note ON
	If AbortPlay = True Then Exit Sub
	Call Offit(note)                         ' Stop that same note
	
	Next                                          ' Play next note

	If AbortPlay = True Then Exit Sub
	Delay 6000                                 ' Pause between scales
	

	If AbortPlay = True Then Exit Sub
	For note = LastNote To FirstNote Step -1   ' Descending Scales

	
	  Call Playit(note)                        ' Play a note
	  If AbortPlay = True Then Exit Sub
	  Delay 7000                               ' Duration of note ON

	  Call Offit(note)                         ' Stop that same note

	Next

	If AbortPlay = True Then Exit Sub
	Delay 7000                                ' Pause between demos


	Form1.text1.text = " MPUDEMO1 now playing some chords on MIDI Channel " + Str$(MidiChannel)

	If AbortPlay = True Then Exit Sub
	For n = 1 To 3                          ' Playing first chord thrice
	
	If AbortPlay = True Then Exit Sub
		note = 65                       ' F3
		Call Playit(note)               ' Start a chord
		note = 69                       ' A3
		Call Playit(note)
		note = 72                       ' C4
		Call Playit(note)

		Delay 9000                     ' Duration of held chord

		note = 65                       ' F3
		Call Offit(note)                ' Stop the chord
		note = 69                       ' A3
		Call Offit(note)
		note = 72                       ' C4
		Call Offit(note)

		Delay 9000                     ' Duration of rest


	Next                                    ' Play chord again
	
	If AbortPlay = True Then Exit Sub
		NoteVelocity = 127
		note = 64                       ' E3
		Call Playit(note)               ' Start last chord
		note = 67                       ' G3
		Call Playit(note)
		note = 72                       ' C4
		Call Playit(note)

		Delay 18000                     ' Duration of held chord

		note = 64
		Call Offit(note)                ' Stop the chord
		note = 67
		Call Offit(note)
		note = 72
		Call Offit(note)
			       
	Form1.text1.text = "       MPUDEMO1 is through - Tinker with it!"

End Sub

' ***************************** Playit SUBROUTINE ***************************
Sub Playit (note As Integer)           ' Play a MIDI Note
    

	OUT DataPort, MidiChannel + 143         ' Send NoteOn with Channel X
	a = INP(DataPort)                       ' Dummy Read to clear buffer *
	MpuWait statport, DRR, maskflip         ' Wait for port ready
	
	OUT DataPort, note                      ' Send note Number to turn ON
	a = INP(DataPort)                       ' Dummy Read to clear buffer *
	MpuWait statport, DRR, maskflip         ' Wait for port ready

	OUT DataPort, NoteVelocity              ' Send medium velocity
	a = INP(DataPort)                       ' Dummy Read to clear buffer *
	MpuWait statport, DRR, maskflip         ' Wait for port ready

End Sub

Sub Offit (note As Integer)               ' Turn off a MIDI Note

'****************************** Offit routine ******************************
' * Note: Read of DataPort prevents hang-up if MIDI IN from a keyboard is
'   connected and played - MpuWait would stay FOREVER if you hit any key once!

	
	OUT DataPort, MidiChannel + 143         ' Send NoteOn with Channel X
	a = INP(DataPort)                       ' Dummy Read to clear buffer *
	MpuWait statport, DRR, maskflip         ' Wait for port ready

	OUT DataPort, note                      ' Send note number to turn OFF
	a = INP(DataPort)                       ' Dummy Read to clear buffer *
	MpuWait statport, DRR, maskflip         ' Wait for port ready

	OUT DataPort, NoteOff                   ' Send 0 Velocity = Note Off
	a = INP(DataPort)                       ' Dummy Read to clear buffer *
	MpuWait statport, DRR, maskflip         ' Wait for port ready

End Sub

Sub Delay (count)
  For x = 1 To count
  'x = DoEvents()
  Next x
End Sub

' ************************** MpuWait subroutine **********************************
' * This routine reads the statport, xor's the data with maskflip (0FFH) and  *
' * ANDs it with DRR or DSR (MpuData).                                        *
' *****************************************************************************
'
Sub MpuWait (statport, MpuData, maskflip)
  count = 0
  Statportbyte = INP(statport)                              ' Get any data at midi statport
  x = DoEvents()
  While ((Statportbyte Xor maskflip) And MpuData) = False   ' Loop until either bit 6 or 7 (DRR or DSR) are set
    If AbortPlay = -1 Then Exit Sub
    Statportbyte = INP(statport)                            ' Get data again if necessary
    count = count + 1
    If count = 256 Then
      Beep: Beep: Beep
      MsgBox "MPU Not Available - program AbortPlayed", 16, "MPU Reset"
      End
    End If
  Wend
End Sub

